// 
// quix-interface
// (c) 1997 geisler@primus.baynet.de
// (c) 1997 biber@rz.fh-augsburg.de
// (c) 1997 wohlhuet@rz.fh-augsburg.de
//

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#ifndef WIN32
	#include <strings.h>
	#include <ctype.h>
 #endif

#include "quixernet.h"
#include "quixer.h"


FILE* fLog;


void Lower(char* text)
{
	while (*text != '\0')
	{
		*text = tolower(*text);
		text++;
	}
}


void CleanFromRight(char* text)
{
	int length = strlen(text);
	if (text[length - 1] == '\n')
	{
		text[length - 1] = '\0';
	}
}


void CutOffAlias(char* dest, char* source)
{
	char* endpos;
	char* startpos;

	// EMail-Adresse in spitzen Klammern?
	// z.B. "Thomas Wohlhter" <wohlhuet@rz.fh-augsburg.de>
	//      Thomas Wohlhter <wohlhuet@rz.fh-augsburg.de>
	//      "'wohlhuet@rz.fh-augsburg.de'" <wohlhuet@rz.fh-augsburg.de>
	startpos = strchr(source, '<');
	if (startpos != NULL)
	{
		endpos = strchr(startpos, '>');
		if (endpos != NULL)
		{
			*endpos = '\0';
			strcpy(dest, startpos + 1);
			return;
		}
	}

	// EMail-Adresse als erstes Wort
	// z.B. wohlhuet@Informatik.FH-Augsburg.DE (Wohlhueter Thomas)
	//      wohlhuet@informatik.fh-augsburg.de
	endpos = strchr(source, ' ');
	if (endpos != NULL)
	{
		*endpos = '\0';
	}
	strcpy(dest, source);
	CleanFromRight(dest);	// \n entfernen
}


BOOL ParseMail(char* from, char* to, char* subject)
{
	char line[1000];
	BOOL bFoundFrom = FALSE;
	BOOL bFoundTo = FALSE;
	BOOL bFoundSubject = FALSE;

	while (!feof(stdin) && !(bFoundFrom && bFoundTo && bFoundSubject))
	{
		fgets(line, sizeof(line), stdin);
		if (strncmp(line, FROM_KEY, strlen(FROM_KEY)) == 0)
		{
			CutOffAlias(from, line + strlen(FROM_KEY));
			Lower(from);
			bFoundFrom = TRUE;
		}
		else
		if (strncmp(line, TO_KEY, strlen(TO_KEY)) == 0)
		{
			CutOffAlias(to, line + strlen(TO_KEY));
			Lower(to);
			bFoundTo = TRUE;
		}
		else
		if (strncmp(line, SUBJECT_KEY, strlen(SUBJECT_KEY)) == 0)
		{
			strcpy(subject, line + strlen(SUBJECT_KEY));
			CleanFromRight(subject);
			bFoundSubject = TRUE;
		}
	}

	// Logfile weiterschreiben
	if (bFoundFrom)
		fprintf(fLog, "From:    %s\n", from);
	else
		fprintf(fLog, "ERROR: Sender not found\n");

	if (bFoundTo)
		fprintf(fLog, "To:      %s\n", to);
	else
		fprintf(fLog, "ERROR: Recipient not found\n");

	if (bFoundSubject)
		fprintf(fLog, "Subject: %s\n", subject);
	else
		fprintf(fLog, "ERROR: Subject not found\n");

	return (bFoundFrom && bFoundTo && bFoundSubject);
}


BOOL GetNumber(char* from, char* to, char* number, char* carrier, char* nick)
{
	FILE* fConfig;
	char line[255];
	char name[255];
	char send[5];
	BOOL bFoundTo = FALSE;
	BOOL bFoundFrom = FALSE;
	BOOL bSectionChange;
	char unknownNick[255];
	BOOL bSendFromUnknown = FALSE;
	
	// Konfigurationsdatei oeffnen
	if ((fConfig = fopen(CONFIGFILE, "r")) == NULL) 
	{
		fprintf(fLog, "ERROR: Can't open configuration file '%s'\n", CONFIGFILE);
		return FALSE;
	};
	
	while (!feof(fConfig) && !(bFoundFrom && bFoundTo))
	{
		bSectionChange = FALSE;

		fgets(line, sizeof(line), fConfig);
		if (strncmp(line, GLOBALSECTION, strlen(GLOBALSECTION)) == 0)
		{
			// Empfaenger
			while (!feof(fConfig) && !bFoundTo && !bSectionChange)
			{
				fgets(line, sizeof(line), fConfig);
				if (line[0] == '#')
					// Sektionswechsel
					bSectionChange = TRUE;
				else
				if (sscanf(line, "%s %s %s", name, number, carrier) == 3)
				{
					// Zeile mit drei Eintraegen gefunden
					// Eintrag fuer Empfaenger gefunden?
					bFoundTo = (strcmp(name, to) == 0);
				}
			}
		}
		else
		if (strncmp(line, KNOWNSECTION, strlen(KNOWNSECTION)) == 0)
		{
			// Sender
			while (!feof(fConfig) && !bFoundFrom && !bSectionChange)
			{
				fgets(line, sizeof(line), fConfig);
				if (line[0] == '#')
					// Sektionswechsel
					bSectionChange = TRUE;
				else
				if (sscanf(line, "%s %s %s", name, nick, send) == 3)
				{
					// Zeile mit drei Eintraegen gefunden
					if (strcmp(name, UNKNOWNSENDER) == 0)
					{
						// Default-Eintrag gefunden
						strcpy(unknownNick, nick);
						bSendFromUnknown = (strcmp(send, "y") == 0);
					}
					else
						// Eintrag fuer Absender gefunden?
						bFoundFrom = (strcmp(name, from) == 0);
				}
			}
		}
	}

	fclose(fConfig);

	if (!bFoundTo)
	{
		fprintf(fLog, "ERROR: Unknown recipient\n");
		return FALSE;
	}

	if (!bFoundFrom)
	{
		// Eintrag des Absenders nicht gefunden
		if (bSendFromUnknown)
		{
			// Nickname fuer unbekannten Absender uebernehmen
			strcpy(nick, from);
		}
		else
		{
			fprintf(fLog, "Error: Unknown sender\n");
			return FALSE;
		}
	}
	else
	{
		// Eintrag des Absenders wurde gefunden
		if (strcmp(send, "y") != 0)
		{
			fprintf(fLog, "Messages from this sender will not be transmitted!\n");
			return FALSE;
		}
	}

	fprintf(fLog, "Carrier: %s, No# %s\n", carrier, number);
	return TRUE;
}

BOOL BuildMessage(char* message, char* nick, char* text)
{
	sprintf(message, "%s: %s", nick, text);
	// Laenge der Nachricht begrenzen
	message[MESSAGELENGTH] = '\0';
	
	fprintf(fLog, "Message: %s\n", message);
	return TRUE;
}


BOOL ExtendMessage(char* output, char* number, char* carrier, char* message)
{
	if (strcmp(carrier, QUIX) == 0)
	{
		sprintf(output, "danke=%%2FQUIX%%2FDANKE.HTM&nr=%s&subject=%s",	number, message); 
	}
	else
	if (strcmp(carrier, D1) == 0)
	{
		sprintf(output, "GET http://%s/cgi-bin/sendsms?serial=3334&error=http%%3A%%2F%%2Fwww.nokia-shop.de%%2Fsmser.htm&next=http%%3A%%2F%%2Fwww.nokia-shop.de%%2Fsmsok.htm&network=%s&target=%s&text=%s&cemail=&validity=0 HTTP/1.0",
				D1_HOST, carrier, number, message); 
	}
	else
	if (strcmp(carrier, D2) == 0)
	{
		sprintf(output, "GET http://%s/cgi-bin/sendsms?serial=3334&error=http%%3A%%2F%%2Fwww.nokia-shop.de%%2Fsmser.htm&next=http%%3A%%2F%%2Fwww.nokia-shop.de%%2Fsmsok.htm&network=%s&target=%s&text=%s&cemail=&validity=0 HTTP/1.0",
				D2_HOST, carrier, number, message);
	}
	else
	{
		fprintf(fLog, "ERROR: Unknown carrier '%s'\n", carrier);
		return FALSE;
	}

	return TRUE;
}


BOOL SendMessage(char* text, char* carrier)
{
	char temp[255];
	int i;

	if (strcmp(carrier, QUIX) == 0)
	{
		if ((i = net_connect(80, QUIX_HOST)) != 0)
		{
			net_sendline(i, "POST /cgi-bin/quixmail HTTP/1.0\n");
			net_sendline(i, "User-Agent: Mozilla/3.01Gold (X11; I; SunOS 5.5 sun4m)\n");
			net_sendline(i, "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*\n");
			net_sendline(i, "Content-type: application/x-www-form-urlencoded\n");
			sprintf(temp, "Content-length: %d\n", strlen(text));
			net_sendline(i, temp);
			sprintf(temp, "Host: %s\n", QUIX_HOST);
			net_sendline(i, temp);
			sprintf(temp, "Referer: http://%s/quix.html\n\n", QUIX_HOST);
			net_sendline(i, temp);
			net_sendline(i, text);
			net_sendline(i, "\n");
		}
		else
		{
			fprintf(fLog, "ERROR: Can't get connection to Host '%s'\n", QUIX_HOST);
			return FALSE;
		}
	}
	else
	if (strcmp(carrier, D1) == 0)
	{
		if ((i = net_connect(80, D1_HOST)) != 0)
		{
			net_sendline(i, text);
			net_sendline(i, "\n");
			net_sendline(i, "Referer: http://WWW.NOKIA-SHOP.DE/sms.htm\n");
			net_sendline(i, "Proxy-Connection: Keep-Alive\n");
			net_sendline(i, "User-Agent: Mozilla/4.03 [en] (X11; I; Linux 2.1.63 CrayII)\n");
			sprintf(temp, "Host: %s\n", D1_HOST);
			net_sendline(i, temp);
			net_sendline(i, "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*\n");
			net_sendline(i, "Accept-Language: de\n");
			net_sendline(i, "Accept-Charset: iso-8859-1,*,utf-8\n");
			net_sendline(i, "\n");
		}
		else
		{
			fprintf(fLog, "ERROR: Can't get connection to Host '%s'\n", D1_HOST);
			return FALSE;
		}
	}
	else 
	if (strcmp(carrier, D2) == 0)
	{
		if ((i = net_connect(80, D2_HOST)) != 0)
		{
			net_sendline(i, text);
			net_sendline(i, "\n");
			net_sendline(i, "Referer: http://WWW.NOKIA-SHOP.DE/sms.htm\n");
			net_sendline(i, "Proxy-Connection: Keep-Alive\n");
			net_sendline(i, "User-Agent: Mozilla/4.03 [en] (X11; I; Linux 2.1.63 CrayII)\n");
			sprintf(temp, "Host: %s\n", D2_HOST);
			net_sendline(i, temp);
			net_sendline(i, "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*\n");
			net_sendline(i, "Accept-Language: de\n");
			net_sendline(i, "Accept-Charset: iso-8859-1,*,utf-8\n");
			net_sendline(i, "\n");
		}
		else
		{
			fprintf(fLog, "ERROR: Can't get connection to Host '%s'\n", D2_HOST);
		}
	}
	else
	{
		fprintf(fLog, "ERROR: Unknown carrier '%s'\n", carrier);
		return FALSE;
	}

	// Was macht denn diese Funktion ??????
	while (net_getline(i, temp, 515))
	{
	}

	fprintf(fLog, "Dispatch OK\n");
	return TRUE;
};


void main(int argc, char* argv[])
{
	time_t zeit;
	char from[255];
	char to[255];
	char subject[255];
	char message[255];
	char output[255];
	char number[20];
	char carrier[20];
	char nick[255];

	// Protokolldatei ffnen
	if ((fLog = fopen(LOGFILE, "a")) == NULL) 
	{
		printf("ERROR: Can't open logfile '%s'\n", LOGFILE);
	}
	
	// Neuen Abschnitt in der Protokolldatei mit Zeitstempel beginnen
	time(&zeit);
	fprintf(fLog, "---------------\n");
	fprintf(fLog, "Time: %s", asctime(localtime(&zeit)));

	// Parsen der EMail nach Sender, Empfaenger und Betreff
	if (!ParseMail(from, to, subject))
	{
	}
	else
	// EMail-Adressen in Konfigurationsdatei suchen
	if (!GetNumber(from, to, number, carrier, nick))
	{
	}
	else
	// Nachricht zusammenbauen
	if (!BuildMessage(message, nick, subject))
	{
	}
	else
	// Nachricht je nach Dienst mit Informationen erweitern
	if (!ExtendMessage(output, number, carrier, message))
	{
	}
	else
	{
		// Nachricht senden
#ifdef _DEBUG
	fprintf(fLog, "Output:  %s\n", output);
#endif
	SendMessage(output, carrier);
	}

	// Protokolldatei schliessen
	fclose(fLog);
};
